@@ -6,6 +6,7 @@
 #include "output_model.h"
 
 #include "../common/utils.h"
+#include "kscreen/edid.h"
 
 #include "config_handler.h"
 
@@ -34,8 +35,17 @@ QVariant OutputModel::data(const QModelIndex &index, int role) const
 
     const KScreen::OutputPtr &output = m_outputs[index.row()].ptr;
     switch (role) {
-    case Qt::DisplayRole:
-        return Utils::outputName(output);
+    case Qt::DisplayRole: {
+        bool shouldShowSerialNumber = false;
+        if (output->edid()) {
+            shouldShowSerialNumber = std::any_of(m_outputs.cbegin(), m_outputs.cend(), [output](const OutputModel::Output &other) {
+                return other.ptr->id() != output->id() // avoid same output
+                    && other.ptr->edid() && other.ptr->edid()->name() == output->edid()->name() // model
+                    && other.ptr->edid()->vendor() == output->edid()->vendor();
+            });
+        }
+        return Utils::outputName(output, shouldShowSerialNumber);
+    }
     case EnabledRole:
         return output->isEnabled();
     case InternalRole:
@@ -60,6 +70,8 @@ QVariant OutputModel::data(const QModelIndex &index, int role) const
         return resolutionIndex(output);
     case ResolutionsRole:
         return resolutionsStrings(output);
+    case ResolutionRole:
+        return resolution(output);
     case RefreshRateIndexRole:
         return refreshRateIndex(output);
     case ReplicationSourceModelRole:
@@ -70,7 +82,8 @@ QVariant OutputModel::data(const QModelIndex &index, int role) const
         return replicasModel(output);
     case RefreshRatesRole: {
         QVariantList ret;
-        for (const auto rate : refreshRates(output)) {
+        const auto rates = refreshRates(output);
+        for (const auto rate : rates) {
             ret << i18n("%1 Hz", int(rate + 0.5));
         }
         return ret;
@@ -136,6 +149,10 @@ bool OutputModel::setData(const QModelIndex &index, const QVariant &value, int r
             return setRefreshRate(index.row(), value.toInt());
         }
         break;
+    case ResolutionRole:
+        // unimplemented
+        return false;
+        break;
     case AutoRotateRole:
         if (value.canConvert<bool>()) {
             return setAutoRotate(index.row(), value.value<bool>());
@@ -160,10 +177,13 @@ bool OutputModel::setData(const QModelIndex &index, const QVariant &value, int r
         bool ok;
         const qreal scale = value.toReal(&ok);
         if (ok && !qFuzzyCompare(output.ptr->scale(), scale)) {
-            const auto oldSize = output.ptr->logicalSize().toSize();
+            const auto oldSize = output.ptr->explicitLogicalSize().toSize();
+
             output.ptr->setScale(scale);
             m_config->setScale(output.ptr, scale);
-            const auto newSize = output.ptr->logicalSize().toSize();
+
+            const auto newSize = m_config->config()->logicalSizeForOutput(*output.ptr).toSize();
+            output.ptr->setExplicitLogicalSize(newSize);
 
             maintainSnapping(output, oldSize, newSize);
 
@@ -230,6 +250,7 @@ QHash<int, QByteArray> OutputModel::roleNames() const
     roles[ScaleRole] = "scale";
     roles[ResolutionIndexRole] = "resolutionIndex";
     roles[ResolutionsRole] = "resolutions";
+    roles[ResolutionRole] = "resolution";
     roles[RefreshRateIndexRole] = "refreshRateIndex";
     roles[RefreshRatesRole] = "refreshRates";
     roles[ReplicationSourceModelRole] = "replicationSourceModel";
@@ -245,7 +266,7 @@ QHash<int, QByteArray> OutputModel::roleNames() const
 void OutputModel::add(const KScreen::OutputPtr &output)
 {
     const int insertPos = m_outputs.count();
-    Q_EMIT beginInsertRows(QModelIndex(), insertPos, insertPos);
+    beginInsertRows(QModelIndex(), insertPos, insertPos);
 
     int i = 0;
     while (i < m_outputs.size()) {
@@ -270,7 +291,7 @@ void OutputModel::add(const KScreen::OutputPtr &output)
     connect(output.data(), &KScreen::Output::isPrimaryChanged, this, [this, output]() {
         roleChanged(output->id(), PrimaryRole);
     });
-    Q_EMIT endInsertRows();
+    endInsertRows();
 
     connect(output.data(), &KScreen::Output::modesChanged, this, [this, output]() {
         rolesChanged(output->id(), {ResolutionsRole, ResolutionIndexRole, SizeRole});
@@ -296,9 +317,9 @@ void OutputModel::remove(int outputId)
     });
     if (it != m_outputs.end()) {
         const int index = it - m_outputs.begin();
-        Q_EMIT beginRemoveRows(QModelIndex(), index, index);
+        beginRemoveRows(QModelIndex(), index, index);
         m_outputs.erase(it);
-        Q_EMIT endRemoveRows();
+        endRemoveRows();
     }
 }
 
@@ -306,7 +327,7 @@ void OutputModel::resetPosition(const Output &output)
 {
     if (output.posReset.x() < 0) {
         // KCM was closed in between.
-        for (const Output &out : m_outputs) {
+        for (const Output &out : qAsConst(m_outputs)) {
             if (out.ptr->id() == output.ptr->id()) {
                 continue;
             }
@@ -384,9 +405,12 @@ bool OutputModel::setResolution(int outputIndex, int resIndex)
     if (output.ptr->currentModeId() == id) {
         return false;
     }
-    const auto oldSize = output.ptr->logicalSize().toSize();
+    const auto oldSize = output.ptr->explicitLogicalSize().toSize();
     output.ptr->setCurrentModeId(id);
-    const auto newSize = output.ptr->logicalSize().toSize();
+    output.ptr->setSize(output.ptr->currentMode()->size());
+
+    const auto newSize = m_config->config()->logicalSizeForOutput(*output.ptr).toSize();
+    output.ptr->setExplicitLogicalSize(newSize);
 
     maintainSnapping(output, oldSize, newSize);
 
@@ -465,9 +489,11 @@ bool OutputModel::setRotation(int outputIndex, KScreen::Output::Rotation rotatio
     if (output.ptr->rotation() == rotation) {
         return false;
     }
-    const auto oldSize = output.ptr->logicalSize().toSize();
+    const auto oldSize = output.ptr->explicitLogicalSize().toSize();
     output.ptr->setRotation(rotation);
-    const auto newSize = output.ptr->logicalSize().toSize();
+
+    const auto newSize = m_config->config()->logicalSizeForOutput(*output.ptr).toSize();
+    output.ptr->setExplicitLogicalSize(newSize);
 
     maintainSnapping(output, oldSize, newSize);
 
@@ -496,6 +522,17 @@ int OutputModel::resolutionIndex(const KScreen::OutputPtr &output) const
     return it - sizes.begin();
 }
 
+QSize OutputModel::resolution(const KScreen::OutputPtr &output) const
+{
+    const QSize currentResolution = output->enforcedModeSize();
+
+    if (!currentResolution.isValid()) {
+        return QSize();
+    }
+
+    return currentResolution;
+}
+
 int OutputModel::refreshRateIndex(const KScreen::OutputPtr &output) const
 {
     if (!output->currentMode()) {
@@ -524,7 +561,8 @@ static int greatestCommonDivisor(int a, int b)
 QVariantList OutputModel::resolutionsStrings(const KScreen::OutputPtr &output) const
 {
     QVariantList ret;
-    for (const QSize &size : resolutions(output)) {
+    const auto resolutionList = resolutions(output);
+    for (const QSize &size : resolutionList) {
         int divisor = greatestCommonDivisor(size.width(), size.height());
 
         if (size.height() / divisor == 5) { // Prefer "16:10" over "8:5"
@@ -550,7 +588,8 @@ QVector<QSize> OutputModel::resolutions(const KScreen::OutputPtr &output) const
 {
     QVector<QSize> hits;
 
-    for (const auto &mode : output->modes()) {
+    const auto modes = output->modes();
+    for (const auto &mode : modes) {
         const QSize size = mode->size();
         if (!hits.contains(size)) {
             hits << size;
@@ -582,7 +621,8 @@ QVector<float> OutputModel::refreshRates(const KScreen::OutputPtr &output) const
         return hits;
     }
 
-    for (const auto &mode : output->modes()) {
+    const auto modes = output->modes();
+    for (const auto &mode : modes) {
         if (mode->size() != baseSize) {
             continue;
         }
@@ -649,7 +689,7 @@ bool OutputModel::setReplicationSourceIndex(int outputIndex, int sourceIndex)
             return false;
         }
         m_config->setReplicationSource(output.ptr, nullptr);
-        output.ptr->setLogicalSize(QSizeF());
+        output.ptr->setExplicitLogicalSize(QSizeF());
         resetPosition(output);
     } else {
         const auto source = m_outputs[sourceIndex].ptr;
@@ -660,7 +700,7 @@ bool OutputModel::setReplicationSourceIndex(int outputIndex, int sourceIndex)
         m_config->setReplicationSource(output.ptr, source);
         output.posReset = output.ptr->pos();
         output.ptr->setPos(source->pos());
-        output.ptr->setLogicalSize(source->logicalSize());
+        output.ptr->setExplicitLogicalSize(m_config->config()->logicalSizeForOutput(*source));
     }
 
     reposition();
@@ -748,7 +788,7 @@ void OutputModel::reposition()
     int y = 0;
 
     // Find first valid output.
-    for (const auto &out : m_outputs) {
+    for (const auto &out : qAsConst(m_outputs)) {
         if (positionable(out)) {
             x = out.ptr->pos().x();
             y = out.ptr->pos().y();
@@ -995,7 +1035,7 @@ bool snapVertical(const QRect &target, const QSize &size, QPoint &dest)
 void OutputModel::snap(const Output &output, QPoint &dest)
 {
     const QSize size = output.ptr->geometry().size();
-    for (const Output &out : m_outputs) {
+    for (const Output &out : qAsConst(m_outputs)) {
         if (out.ptr->id() == output.ptr->id()) {
             // Can not snap to itself.
             continue;
